/*
 * (c) Copyright 2002 by Rob Braun
 * All rights reserved.  The file named COPYRIGHT specifies the terms
 * and conditions for redistribution.
 */

#include "xmodule.h"
#include "service.h"
#include "server.h"
#include "sconf.h"
#include "pset.h"
#include "xdl.h"
#include "xconfig.h"
#include "msg.h"
#include "connection.h"

status_e xinetd_check_module( struct service * sp, const char *func, connection_s *cp )
{
	struct service_config *scp;
	struct module_conf *mc;
	int i;
	char mpath[1024];
	void *handle;
	servicefn callfn;

	if( sp == NULL )
		return FAILED;
	if( func == NULL )
		return FAILED;

	scp = SVC_CONF(sp);
	
	for( i = 0; i < pset_count(scp->sc_modules); i++ ) {
		mc = pset_pointer(scp->sc_modules, i);
		if( mc == NULL )
			continue;
		memset(mpath, 0, sizeof(mpath));
		strcpy(mpath, MODULE_PATH);
		strcat(mpath, mc->name);
		handle = xdlopen(mpath);
		if( handle == NULL ) {
			msg(LOG_ERR, "xinetd_check_module", "Can't find module %s", mpath);
			continue;
		}

		callfn = xdlsym(handle, func);
		if( callfn == NULL ) {
			if( debug.on )
				msg(LOG_DEBUG, "module %s doesn't define %s", mpath, func);
			continue;
		}

		if( callfn(sp, mc, cp) == FAILED ) {
			if( debug.on )
				msg(LOG_DEBUG, "module %s failed", mpath);
			xdlclose(handle);
			return FAILED;
		}

		xdlclose(handle);

	}
	return OK;
}
